@@ -20,6 +20,14 @@ impl<'a> TryFrom<Any<'a>> for EndOfContent {
     type Error = Error;
 
     fn try_from(any: Any<'a>) -> Result<EndOfContent> {
+        TryFrom::try_from(&any)
+    }
+}
+
+impl<'a, 'b> TryFrom<&'b Any<'a>> for EndOfContent {
+    type Error = Error;
+
+    fn try_from(any: &'b Any<'a>) -> Result<EndOfContent> {
         any.tag().assert_eq(Self::TAG)?;
         if !any.header.length.is_null() {
             return Err(Error::InvalidLength);
