@@ -15,6 +15,14 @@ impl<'a> TryFrom<Any<'a>> for Null {
     type Error = Error;
 
     fn try_from(any: Any<'a>) -> Result<Null> {
+        TryFrom::try_from(&any)
+    }
+}
+
+impl<'a, 'b> TryFrom<&'b Any<'a>> for Null {
+    type Error = Error;
+
+    fn try_from(any: &'b Any<'a>) -> Result<Null> {
         any.tag().assert_eq(Self::TAG)?;
         if !any.header.length.is_null() {
             return Err(Error::InvalidLength);
@@ -29,6 +37,8 @@ impl CheckDerConstraints for Null {
     }
 }
 
+impl DerAutoDerive for Null {}
+
 impl<'a> Tagged for Null {
     const TAG: Tag = Tag::Null;
 }
@@ -67,6 +77,8 @@ impl<'a> CheckDerConstraints for () {
     }
 }
 
+impl DerAutoDerive for () {}
+
 impl<'a> Tagged for () {
     const TAG: Tag = Tag::Null;
 }
